<?php
namespace Teknisa\FIN\Modules\ContaPagar\Services\Remessa;

use Teknisa\Model\Financeiro\Entities\Cnabpag;
use Financeiro\Util\Util;
use Fiscal\Entities\Boletpagar;
use Financeiro\Entities\Emprbanc;
use Geral\Entities\Tipobaixapag;
use Geral\Entities\Banco;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco\GeraCnabSantanderService;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco\GeraCnabItauService;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco\GeraCnabCaixaService;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco\GeraCnabBradescoService;
use Teknisa\FIN\Modules\ContaPagar\Services\Remessa\Banco\GeraCnabBrasilService;

/**
 * Class GeraCnab
 * @package Teknisa\FIN\Modules\ContaPagar\Services\Remessa
 */
abstract class GeraCnabService {

    const CODIGO_COMPENSACAO_SANTANDER = "033";
    const CODIGO_COMPENSACAO_BRADESCO  = "237";
    const CODIGO_COMPENSACAO_ITAU      = "341";
    const CODIGO_COMPENSACAO_CAIXA     = "104";
    const CODIGO_COMPENSACAO_BRASIL    = '001';

    const EXCLUSAO = 'E';

    protected $nrorg;
    protected $inscricaojuridica;
    protected $nomeempresa;
    protected $tipoinscricao;

    /** @var $contcorr \Geral\Entities\Contcorr  */
    protected $contcorr;
    /** @var $filial \Geral\Entities\Filial */
    protected $filial;
    /** @var $empresa \Geral\Entities\Empresa */
    protected $empresa;
    /** @var $remessa \Financeiro\Entities\Cnabpag */
    protected $remessa;

    /**
     * @param Cnabpag $remessa
     * @return null|GeraCnabSantanderService
     * @throws \Exception
     */
    public static function factory(
        CnabPag $remessa
    ){
        try {
            /** @var Banco $banco */
            $banco = Banco::findOneByNrorgAndCdbanco($remessa->getNrorg(), $remessa->getCdbanco());

            if($banco){
                $codigoCompensacao = $banco->getCdbanccnab();
                if ($codigoCompensacao === self::CODIGO_COMPENSACAO_SANTANDER) {
                    return new GeraCnabSantanderService($remessa);
                }elseif($codigoCompensacao === self::CODIGO_COMPENSACAO_BRADESCO) {
                    return new GeraCnabBradescoService($remessa);
                }elseif($codigoCompensacao === self::CODIGO_COMPENSACAO_ITAU) {
                    return new GeraCnabItauService($remessa);
                }elseif($codigoCompensacao === self::CODIGO_COMPENSACAO_CAIXA)
                    return new GeraCnabCaixaService($remessa);
                elseif($codigoCompensacao === self::CODIGO_COMPENSACAO_BRASIL)
                    return new GeraCnabBrasilService($remessa);
                else throw new \Exception('O Códido de Compensação informado não pertece a nenhum banco reconhecido.');
            }else return null;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * @return mixed
     */
    public abstract function geraHeaderArquivo();

    /**
     * @param $cdbanco
     * @internal param $formaLancamento
     * @return mixed
     */
    public abstract function geraHeaderLote($cdbanco, $boletos = null);

    /**
     * @return mixed
     */
    public abstract function geraTraillerLote($somavalores = null);

    /**
     * @return mixed
     */
    public abstract function geraTrailerArquivo();

    /**
     * @param $caminhoArquivo
     * @param $boletos
     * @param bool $autorizaAgendamento - Parámetro utilizando quando o banco for Bradesco
     * @internal param $remessa
     * @return mixed
     */
    public abstract function geraArquivo($caminhoArquivo, $boletos, $autorizaAgendamento = false, $idopecnabpag = null);

    /**
     * @param $string
     * @return mixed
     */
    protected function substitui_Sym_Chr(&$string){
        $sym = array(
            'À','Á','Ã','Â', 'à','á','ã','â',
            'Ê', 'É',
            'Í', 'í',
            'Ó','Õ','Ô', 'ó', 'õ', 'ô',
            'Ú','Ü',
            'Ç', 'ç',
            'é','ê',
            'ú','ü',
            'º', '&'
        );
        $chr = array(
            chr(192), chr(193), chr(195), chr(194), chr(224), chr(225), chr(227), chr(226),
            chr(202), chr(201),
            chr(205), chr(237),
            chr(211), chr(213), chr(212), chr(243), chr(245), chr(244),
            chr(218), chr(220),
            chr(199), chr(231),
            chr(233), chr(234),
            chr(250), chr(252),' ',' '
        );
        return str_replace($sym, $chr, $string);
    }

    /**
     * @return null
     */
    protected  function getIdentPag(){
        if($this->contcorr->getCdidpagcontcorr())
            return $this->contcorr->getCdidpagcontcorr();
        else{
            /** @var $emprbanc Emprbanc*/
            $emprbanc = Emprbanc::findOneByCdempresaAndCdbancoAndNrorg($this->empresa->getCdempresa(),
                $this->remessa->getCdbanco(),
                $this->nrorg
            );
            if($emprbanc and $emprbanc->getCdempbancpag())
                return $emprbanc->getCdempbancpag();
            else
                return null;
        }
    }

    /**
     * @param Boletpagar $boleto
     * @param null $idbaixautilp
     * @return string
     */
    protected function getBanco(Boletpagar $boleto, $idbaixautilp = null){
        if ($idbaixautilp == Tipobaixapag::TIPO_BAIXA_CNAB_BOLETO){
            return substr($boleto->getCdbarrapag(),0,3);
        }else{
            if($boleto->getCdbancbolp())
                return $boleto->getCdbancbolp();
            else
                return $boleto->getReducont()->getCdbanco();
        }
    }

    /**
     * @param $cdbanco
     * @param $nrorg
     * @return mixed
     */
    protected function getBancoForn($cdbanco, $nrorg){
        $banco = Banco::findOneByCdbanccnabAndNrorg($cdbanco, $nrorg);
        return $banco;
    }

    /** Retorna Conta Corrente sem a ultima posição (dígito)
     * @return string
     */
    protected function getContaCorrenteSemDigito(){
        $contacorrente = $this->contcorr->getCdcontcorr();
        return substr($contacorrente,0,-1);
    }

    /**
     * Retorna a ultima posição da Conta Corrente (Dígito)
     * @return int|string
     */
    protected function getDigitoContaCorrente(){
        $contacorrente = $this->contcorr->getCdcontcorr();
        return substr($contacorrente,-1);
    }

}