<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\Bco11100TransferenciaContaCorrenteFactory;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Services\ContabilizacaoService;

class Bco11100TransferenciaContaCorrenteService
{
    protected $environment;
    protected $newCode;
    protected $entityManager;
    protected $factory;
    protected $fluxodecaixaservice;
    public $nrmovbanc;
    protected $contabilizacaoService;
    protected $nrtranbanc;
    const ENTRADA    = 'E';
    const SAIDA      = 'S';
    const LONGOPRAZO = 'L';
    const DEBITO     = 'D';
    const MOV_FLUXO_REAL     = 'R';

    public function __construct($environment, $newCode, EntityManager $entityManager,
                                Bco11100TransferenciaContaCorrenteFactory $factory,
                                FluxoDeCaixaService $fluxodecaixaservice,
                                ContabilizacaoService $contabilizacaoService) {
        $this->environment           = $environment;
        $this->newCode               = $newCode;
        $this->entityManager         = $entityManager;
        $this->factory               = $factory;
        $this->fluxodecaixaservice   = $fluxodecaixaservice;
        $this->contabilizacaoService = $contabilizacaoService;
        $this->nrorg                 = $this->environment->getNrOrgTrab();
    }

    public function saveTransferenciaSaldo($transferencia, $itens, $nrorg, $response, $contabiliza){
        $this->validaCondicoesParaTransferência($nrorg, $itens);
        $this->nrtranbanc = $this->geraCodigo('TRANSFBANC' . $transferencia['CDEMPRESA'], 'NRTRANBANC');
        $this->insereTransferenciaSaldo($transferencia);
        foreach ($itens as $item) {
            $this->insereItensTransferencia(
                $item['CDEMPRESA'], $nrorg, $item['DTTRANBANC'], $item['DSHISTTBANC'],
                $item['VRMOVITBANC'], $item['CDREDUCONT'], $item['IDMOVBANC'], $this->nrtranbanc
            );
        }
        if($contabiliza == true){
            $this->validaContabilizacao(
                $transferencia['CDEMPRESA'],
                $this->nrtranbanc,
                $transferencia['DTTRANBANC']->format('d/m/Y'),
                $this->environment->getCdfilial(),
                $response
            );
        }
        
        $this->atualizaFluxoContaLongoPrazo($transferencia, $itens);
    }

    public function deleteTransferenciaContaCorrente($transferencia, $itens){
        $this->validaCondicoesParaExclusao($transferencia, $itens);
        $this->deleteLancamentoContabil($transferencia);
        $this->deleteItensTransferencia($transferencia);
        $this->atualizaSaldo($itens);
        $this->deleteTransferenciaSaldo($transferencia);
        $this->atualizaFluxo($transferencia);
    }

    public function editaTransferenciaSaldo($transferencia, $itens, $nrorg, $response, $contabiliza){
        foreach ($itens as $item) {
            $this->validaClassificacaoFinanceira(
                $item['CDREDUCONT'], $item['CDEMPRESA'], $nrorg
            );
        }
        $transferencia['VRMOVITBANC'] = floatval($transferencia['VRMOVITBANC']);
        $this->updateTransferenciaSaldo(
            $transferencia['CDEMPRESA'], $nrorg, $transferencia['DSHISTTBANC'],
            $transferencia['VRMOVITBANC'], $transferencia['NRTRANBANC']
        );
        foreach($itens as $row) {
            $row['VRMOVITBANC'] = floatval($row['VRMOVITBANC']);
            $row['DIFERENCASALDO'] = floatval($row['DIFERENCASALDO']);
            if ($row['__is_new']) {
                $this->insereItensTransferencia(
                    $row['CDEMPRESA'], $nrorg, $row['DTTRANBANC'], $row['DSHISTTBANC'],
                    $row['VRMOVITBANC'], $row['CDREDUCONT'], $row['IDMOVBANC'], $row['NRTRANBANC']
                );
            }else if ($row['changed']){
                $this->updateItensTransferencia(
                    $row['CDEMPRESA'], $nrorg, $row['DTTRANBANC'], $row['DSHISTTBANC'],
                    $row['VRMOVITBANC'], $row['CDREDUCONT'], $row['NRTRANBANC'], $row['NRMOVBANC'], $row['DIFERENCASALDO']
                );
            }
        }
        if($contabiliza == true){
            $this->validaContabilizacao(
                $transferencia['CDEMPRESA'],
                $transferencia['NRTRANBANC'],
                $transferencia['DTTRANBANC']->format('d/m/Y'),
                $this->environment->getCdfilial(),
                $response
            );
        }
    }

    public function validaCondicoesParaTransferência($nrorg, $itensTransferencia){
        if (!$this->getHistoricoPadrao($nrorg, 'TRANSF')) {
            throw new \Exception('Histórico padrão para lançamento contábil não definido.');
        }
        foreach ($itensTransferencia as $item) {
            $this->validaClassificacaoFinanceira(
                $item['CDREDUCONT'], $item['CDEMPRESA'], $nrorg
            );
        }
    }

    public function getHistoricoPadrao($nrorg, $cdoperacao){
        return $this->factory->getHistoricoPadrao($nrorg, $cdoperacao);
    }

    public function validaClassificacaoFinanceira($cdreducont, $cdempresa, $nrorg){
        $tipoConta = $this->factory->validaClassificacaoFinanceira($cdreducont, $cdempresa, $nrorg);
        if ($tipoConta == 'L'){
            if (!$this->factory->getClassificacaoFinanceira($nrorg)){
                throw new \Exception('Não existe uma classificação financeira padrão.');
            }
        }
    }

    public function validaCondicoesParaExclusao($transferencia, $itens){
        $this->validaPeriodoContabil($this->nrorg, $transferencia['CDEMPRESA'], $transferencia['DTSDOCCORR']);
        $this->validaAcessoExclusao($transferencia, $itens);
        $this->validaMovimentacaoConciliados($itens);
    }

    public function validaPeriodoContabil($nrorg, $cdempresa, $dtmovbanc){
        $idctbtrans = $this->factory->getIdContabilizacao($nrorg);
        if ($idctbtrans != 'N') {
            if (!$this->factory->validaPeriodoContabil($nrorg, $cdempresa, $dtmovbanc)){
                throw new \Exception('Operação Bloqueada! Data de Transferência fora do Período Contábil.');
            }
        }
    }

    public function validaAcessoOperadorPorContaFilial($cdoperador, $cdfilial, $empresa, $nrorg){
        try {
            if ($cdfilial) {
                if (!$this->factory->validaAcessoFilialRelConta($cdoperador, $cdfilial, $nrorg)) {
                    throw new \Exception('Operação bloqueada. Operador logado não possui acesso a Filial da Conta Corrente informada.');
                }else if (!$this->factory->validaAcessoFilialEmpresa($cdoperador, $empresa, $nrorg)) {
                    throw new \Exception('Operação bloqueada. Operador logado não possui acesso a Filial da Conta Corrente informada.');
                }
            }
        }catch (\Exception $e){
            throw $e;
        }
    }

    public function insereTransferenciaSaldo($transferencia){
        try {

            $transfbanc = $this->factory->insereTransferencia($transferencia, $this->nrorg, $this->nrtranbanc);
            $this->entityManager->persist($transfbanc);
            $this->entityManager->flush();
        }catch (\Exception $e){
            throw $e;
        }
    }

    public function insereItensTransferencia($empresa, $nrorg, $dttranbanc, $dshisttbanc,
                                             $vrtranbanc, $cdreducont, $idmovbanc, $nrtranbanc){
        try {
            $nrmovbanc = $this->geraCodigo('MOVBANCARIO' . $cdreducont, 'NRMOVBANC');
            $this->nrmovbanc = ($this->nrmovbanc) ?  $this->nrmovbanc : $nrmovbanc;
            $movbancariocred = $this->factory->insereMovbancario($nrorg, $dttranbanc, $vrtranbanc,
                $dshisttbanc, $cdreducont, $nrmovbanc, $idmovbanc);
            $this->entityManager->persist($movbancariocred);
            $this->entityManager->flush();
            $ittransfbanc = $this->factory->insereItemTransferencia($nrorg, $empresa, $vrtranbanc,
                $cdreducont, $nrtranbanc, $nrmovbanc, $idmovbanc);
            $this->entityManager->persist($ittransfbanc);
            $vrtranbanc = ($idmovbanc == 'C')? $vrtranbanc : $vrtranbanc * -1;
            $this->fluxodecaixaservice->atualizaSaldoContCorr($cdreducont, $dttranbanc, $nrorg, $vrtranbanc);
            $this->entityManager->flush();
        }catch (\Exception $e){
            throw $e;
        }
    }

    public function validaContabilizacao($cdempresa, $nrtranbanc, $dttranbanc, $cdfiliproc, $response){
        try {
            $this->contabilizacaoService->contabilizaTransferenciaBancaria($cdempresa, $nrtranbanc, $dttranbanc, $cdfiliproc);
        }catch(\Exception $e){
            $this->contabilizacaoService->saveContabilizacaoData($response);
            Throw $e;
        }
    }

    public function updateTransferenciaSaldo($empresa, $nrorg, $dshisttbanc, $vrmovitbanc, $nrtranbanc){
        try {
            $transfbanc = $this->factory->updateTransferencia($nrorg, $empresa, $nrtranbanc, $dshisttbanc, $vrmovitbanc);
            $this->entityManager->persist($transfbanc);
            $this->entityManager->flush();
        }catch (\Exception $e){
            throw $e;
        }
    }

    public function updateItensTransferencia($empresa, $nrorg, $dttranbanc, $dshisttbanc,
                                             $vrtranbanc, $cdreducont, $nrtranbanc, $nrmovbanc, $diferencasaldo){
        try {
            $movbancario = $this->factory->updateMovbancario($nrorg, $dshisttbanc, $cdreducont, $nrmovbanc, $vrtranbanc);
            $this->entityManager->persist($movbancario);
            $this->entityManager->flush();
            $ittransfbanc = $this->factory->updateItemTransferencia($nrorg, $empresa, $nrtranbanc, $cdreducont, $vrtranbanc);
            $this->entityManager->persist($ittransfbanc);
            $this->fluxodecaixaservice->atualizaSaldoContCorr($cdreducont, $dttranbanc, $nrorg, $diferencasaldo);
            $this->entityManager->flush();
        }catch (\Exception $e){
            throw $e;
        }
    }

    /*public function deleteItensTransferencia($cdreducont, $empresa, $nrorg, $nrtranbanc, $nrmovbanc, $dtsdoccorr, $vrtranbanc, $idmovbanc){
        try {
            $itensTransferencia = $this->factory->getIttransfbanc($empresa, $nrtranbanc, $cdreducont, $nrorg);
            $this->entityManager->remove($itensTransferencia);
            $movbancario = $this->factory->getMovbancario($nrmovbanc, $cdreducont, $nrorg);
            if ($movbancario->getNrseqmovreba() && $movbancario->getCdsequexba()){
                $movreal = $this->factory->getMovreal($nrorg, $movbancario->getNrseqmovreba(), $movbancario->getCdsequexba(), $cdreducont);
                $movreal = $this->factory->atualizaMovreal($movreal);
                $this->entityManager->persist($movreal);
            }
            $this->entityManager->remove($movbancario);
            $vrtranbanc = ($idmovbanc == 'C')? $vrtranbanc : $vrtranbanc * -1;
            $this->fluxodecaixaservice->atualizaSaldoContCorr($cdreducont, $dtsdoccorr, $nrorg, $vrtranbanc);
        }catch(\Exception $e){
            throw $e;
        }
    }*/

    public function atualizaSaldo($itens){
        try {
            foreach ($itens as $item) {
                $item['DTSDOCCORR'] = self::formataData($item['DTSDOCCORR']);
                $movbancario = $this->factory->getMovbancario($item['NRMOVBANC'], $item['CDREDUCONT'], $this->nrorg);
                $this->entityManager->remove($movbancario);
                $item['VRMOVITBANC'] = ($item['IDMOVBANC'] == 'C')? $item['VRMOVITBANC'] : $item['VRMOVITBANC'] * -1;
                $this->fluxodecaixaservice->atualizaSaldoContCorr($item['CDREDUCONT'], $item['DTSDOCCORR'], $this->nrorg, $item['VRMOVITBANC']);
            }
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function deleteItensTransferencia($transferencia){
        $itens = $this->factory->getIttransfbanc(
            $transferencia['CDEMPRESA'], $transferencia['NRTRANBANC'], $this->nrorg
        );
        if ($itens) {
            foreach($itens as $item) {
                $this->entityManager->remove($item);
            }
        }
    }

    public function deleteTransferenciaSaldo($transferencia){
        try {
            $transfBancaria = $this->factory->getTransfbanc(
                $transferencia['CDEMPRESA'], $transferencia['NRTRANBANC'], $this->nrorg
            );
            $this->entityManager->remove($transfBancaria);
        }catch(\Exception $e){
            throw $e;
        }
    }

    public function atualizaFluxoContaLongoPrazo($transferencia, $itens){
        $vrfluxo = 0;
        foreach ($itens as $item) {
            $tipoConta = $this->factory->validaClassificacaoFinanceira(
                $item['CDREDUCONT'], $item['CDEMPRESA'], $this->nrorg
            );
            if ($tipoConta == self::LONGOPRAZO){
                if ($item['IDMOVBANC'] == self::DEBITO){
                    $vrfluxo = floatVal($vrfluxo) - floatVal($item['VRMOVITBANC']);
                }else{
                    $vrfluxo = floatVal($vrfluxo) + floatVal($item['VRMOVITBANC']);
                }
            }
        }
        $classificacaoFinanceira = $this->factory->getClassificacaoFinanceira($this->nrorg);
        if ($transferencia['CDEMPRESA'] && $classificacaoFinanceira){
            if ($vrfluxo < 0){
                $movfluxo = $this->fluxodecaixaservice->atualizaFluxoCaixa(
                    $this->nrorg, $transferencia['CDEMPRESA'], $this->environment->getCdfilial(), $classificacaoFinanceira,
                    $transferencia['DTTRANBANC'], $vrfluxo * -1, 'R', self::ENTRADA
                );
            }else{
                $movfluxo = $this->fluxodecaixaservice->atualizaFluxoCaixa(
                    $this->nrorg, $transferencia['CDEMPRESA'], $this->environment->getCdfilial(), $classificacaoFinanceira,
                    $transferencia['DTTRANBANC'], $vrfluxo, 'R', self::SAIDA
                );
            }
            $this->entityManager->persist($movfluxo);
            $transfbanc = $this->factory->getTransfbanc($transferencia['CDEMPRESA'], $this->nrtranbanc, $this->nrorg);
            $transfbanc->setIdmovfluxo(self::MOV_FLUXO_REAL);
            $transfbanc->setDtmovfluxo($transferencia['DTTRANBANC']);
            $transfbanc->setCdclassfina($classificacaoFinanceira);
            $transfbanc->setCdfilial($this->environment->getCdfilial());
            $this->entityManager->persist($transfbanc);
        }
    }

    public function atualizaFluxo($transferencia){
        $itens = $this->factory->getItensContaLongoPrazo(
            $transferencia['CDEMPRESA'], $this->nrorg, $transferencia['NRTRANBANC']
        );
        if (count($itens) > 0) {
            foreach ($itens as $item) {
                if ($item['IDMOVITBANC'] == self::DEBITO) {
                    $movfluxo = $this->fluxodecaixaservice->atualizaFluxoCaixa(
                        $this->nrorg, $transferencia['CDEMPRESA'], $item['CDFILIAL'], $item['CDCLASSFINA'],
                        $item['DTMOVFLUXO'], $item['VRMOVITBANC'] * -1, 'R', self::ENTRADA
                    );
                } else {
                    $movfluxo = $this->fluxodecaixaservice->atualizaFluxoCaixa(
                        $this->nrorg, $transferencia['CDEMPRESA'], $item['CDFILIAL'], $item['CDCLASSFINA'],
                        self::formataData($item['DTMOVFLUXO']), $item['VRMOVITBANC'], 'R', self::SAIDA
                    );
                }
                $this->entityManager->persist($movfluxo);
            }
        }
    }

    public function validaAcessoExclusao($transferencia, $itens){
        try{
            $temFilial = 'N';
            $naoTemFilial = 'N';
            $podeExcluir = 'S';

            $idpertransccor = $this->factory->getGeralBco($transferencia['NRORG']);

            if ($idpertransccor == 'N') {
                $podeExcluir = 'N';
                //verifica se existe filial cadastrada para alguma conta corrente da transferencia
                foreach ($itens as $item) {
                    $conta = $this->factory->getContaCorrente(
                        $transferencia['CDEMPRESA'], $transferencia['NRORG'], $item['CDREDUCONT']
                    );
                    if ($conta[0]['CDFILIAL']){
                        $temFilial = 'S';
                    }else{
                        $naoTemFilial = 'S';
                    }
                }
                //verifica se o operador tem acesso a todas as filiais da empresa, caso nao tenha nao podera excluir a transf.
                if (($temFilial == 'N') || (($temFilial  == 'S') && ($naoTemFilial == 'S'))){
                    $filial = $this->factory->getFilial($this->nrorg, $transferencia['CDEMPRESA']);
                    $filialEmpresa = $this->factory->getFilialEmpresa(
                        $this->environment->getCdoperador(), $transferencia['CDEMPRESA'], $this->nrorg, 'T'
                    );
                    if (count($filial) == count($filialEmpresa)){
                        $podeExcluir = 'S';
                    }
                }
                //verifica se o operador tem acesso as filiais da conta, se tiver acesso a todas parametrizadas permite excluir.
                if (($temFilial == 'S') && ($naoTemFilial == 'N')){

                    foreach ($itens as $item) {
                        $conta = $this->factory->getContaCorrente(
                            $transferencia['CDEMPRESA'], $transferencia['NRORG'], $item['CDREDUCONT']
                        );
                        if ($conta[0]['CDFILIAL']){
                            $filialEmpresa = $this->factory->getFilialEmpresa($this->environment->getCdoperador(), $item['CDEMPRESA'], $this->environment->getNrOrgTrab(), $conta[0]['CDFILIAL']);
                            $podeExcluir = ($filialEmpresa)? 'S': 'N';
                        }
                    }
                }
            }
            if ($podeExcluir != 'S') {
                throw new \Exception('Operador não possui permissão para excluir a transferência.');
            }
            return $podeExcluir;
        }catch(\Exception $e){
            Throw $e;
        }
    }

    public function validaMovimentacaoConciliados($rows){
        try{
            foreach ($rows as $row) {
                $movBancario = $this->factory->getMovbancario($row['NRMOVBANC'], $row['CDREDUCONT'], $row['NRORG']);
                if (($movBancario->getNrseqmovreba() && $movBancario->getCdsequexba()) || ($movBancario->getNrconcimovb())){
                    throw new \Exception('Exclusão não permitida! Movimento bancário da transferência foi conciliado junto com outros movimentos bancários.');
                }
            }
        }catch(\Exception $e){
            Throw $e;
        }
    }

    public function deleteLancamentoContabil($transferencia){
        try{
            $lancamentoContabil = $this->factory->validaLancamentoContabil(
                $this->nrorg, $transferencia['CDEMPRESA'], $transferencia['NRLCTBTRAN']
            );
            if (count($lancamentoContabil) > 0){
                $lancCCusto = $this->factory->validaLancCCusto(
                    $this->nrorg, $transferencia['CDEMPRESA'], $transferencia['NRLCTBTRAN']
                );
                if (count($lancCCusto) > 0){
                    $lancCCusto = $this->factory->getLancCCusto(
                        $this->nrorg, $transferencia['CDEMPRESA'], $transferencia['NRLCTBTRAN']
                    );
                    if ($lancCCusto) {
                        foreach($lancCCusto as $lanc) {
                            $this->entityManager->remove($lanc);
                        }
                    }
                }
                if (count($lancamentoContabil) > 0) {
                    $lancCont = $this->factory->getLancamentoContabil(
                        $this->nrorg, $transferencia['CDEMPRESA'], $transferencia['NRLCTBTRAN']
                    );
                    if ($lancCont) {
                        foreach($lancCont as $lanc) {
                            $this->entityManager->remove($lanc);
                        }
                    }
                }
            }
        }catch(\Exception $e){
            Throw $e;
        }
    }

    public function geraCodigo($cdcontador, $chave){
        $row['__is_new_code'] = true;
        $row['__new_code_pk'] = $cdcontador;
        $row['__table_pk'] = $chave;
        $row['__new_code_length'] = 10;
        $row['__is_new'] = true;
        if(!empty($row['__new_code_pk'])) {
            $castNumber = empty($row['__cast_number']) ? false : $row['__cast_number'];
            $newCodeKey = empty($row['__using_unit']) ? $row['__new_code_pk'] : ($row['__new_code_pk'].$this->environment->getCdFilial());
            $newCodeKey = strtoupper($newCodeKey);
            return $this->newCode->getCodeByTableName($newCodeKey, $this->environment->getNrOrgTrab(), 1, $row['__new_code_length'], $castNumber);
        }
    }

    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}