<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\SaveGeralBcoParamgeralFactory;


class SaveGeralBcoParamgeral {

    protected $entityManager;
    protected $crudService;

    public function __construct($crudService , $entityManager, $environment, SaveGeralBcoParamgeralFactory $saveGeralBcoParamgeralFactory){
        $this->entityManager                 = $entityManager;
        $this->crudService                   = $crudService;
        $this->environment                   = $environment;
        $this->saveGeralBcoParamgeralFactory = $saveGeralBcoParamgeralFactory;
    }

    /*public function doubleSave($banco1, $dataSourceName1, $banco2, $dataSourceName2, $banco3, $dataSourceName3,  $method, $routePath, $userId) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        $paramgeral = $this->saveGeralBcoParamgeralFactory->getParamgeral();
        if ($paramgeral){
            $requestParamGeral = $this->requestSet($banco1, $dataSourceName1, $method, $routePath, $userId);
            $this->saveSet($requestParamGeral, $dataSourceName1, $connection);
        }
        $geralBco = $this->saveGeralBcoParamgeralFactory->getGeralBco();
        if ($geralBco){
            $banco2['CDGERALBCO'] = 1;
            $banco2['NRORG'] = 1;
        }
        $requestGeralBco = $this->requestSet($banco2, $dataSourceName2, $method, $routePath, $userId);
        $this->saveSet($requestGeralBco, $dataSourceName2, $connection);

        $geralFcx = $this->saveGeralBcoParamgeralFactory->getGeralFcx();
        if ($geralBco){
            $requestGeralFcx = $this->requestSet($banco3, $dataSourceName3, $method, $routePath, $userId);
            $this->saveSet($requestGeralFcx, $dataSourceName3, $connection);
        }
        $connection->commit();
    }*/
    
    public function saveParametrosControleBancario($rowParamgeral, $rowGeralbco, $rowGeralfcx) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            $rowParamgeral['NRORG'] = $this->environment->getNrOrgTrab();
            $rowGeralfcx['NRORG']   = $this->environment->getNrOrgTrab();
            $rowGeralbco['NRORG']   = $this->environment->getNrOrgTrab();
            
            // Busca Parâmetro Geral
            $paramgeral = $this->saveGeralBcoParamgeralFactory->getParamgeral();
            
            if ($paramgeral){
                $paramgeral = $this->saveGeralBcoParamgeralFactory->atualizaParamgeral($paramgeral, $rowParamgeral);
                $this->entityManager->persist($paramgeral);
            }else{
                $rowParamgeral['CDPARGERAL'] = 1;
                $paramgeral = $this->saveGeralBcoParamgeralFactory->insereParamgeral($rowParamgeral);
                $this->entityManager->persist($paramgeral);
            }
            
            // Busca Parâmetros Controle Bancário
            $geralBco = $this->saveGeralBcoParamgeralFactory->getGeralBco();
            
            if ($geralBco){
                $geralBco = $this->saveGeralBcoParamgeralFactory->atualizaGeralBco($geralBco, $rowGeralbco);
                $this->entityManager->persist($geralBco);
            }else{
                $rowGeralbco['CDGERALBCO'] = 1;
                $geralBco = $this->saveGeralBcoParamgeralFactory->insereGeralBco($rowGeralbco);
                $this->entityManager->persist($geralBco);
            }
            
            // Busca Parâmetros Fluxo de Caixa
            $geralFcx = $this->saveGeralBcoParamgeralFactory->getGeralFcx();
            
            if ($geralFcx){
                $geralFcx = $this->saveGeralBcoParamgeralFactory->atualizaGeralFcx($geralFcx, $rowGeralfcx);
                $this->entityManager->persist($geralFcx);
            }else{
                $rowGeralfcx['CDGERALFCX'] = 1;
                $geralFcx = $this->saveGeralBcoParamgeralFactory->insereGeralFcx($rowGeralfcx);
                $this->entityManager->persist($geralFcx);
            }
            
            $this->entityManager->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }
    
    public function validaGeralBcoPopulada(){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $geralBco = $this->saveGeralBcoParamgeralFactory->getGeralBco();
            if (!$geralBco){
                $rowGeralbco = array(
                    'CDGERALBCO' => 1,
                    'NRORG'      => $this->environment->getNrOrgTrab()
                );
                $geralBco = $this->saveGeralBcoParamgeralFactory->popularGeralBco($geralBco, $rowGeralbco);
                $this->entityManager->persist($geralBco);
                $this->entityManager->flush();
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function saveSet($request, $dataSourceName, $connection){
       try {
            $rows = $this->crudService->getRowsToSave($request, $dataSourceName);
            $this->crudService->save($dataSourceName, $rows);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function requestSet($row, $dataSourceName, $method, $routePath, $userId){
        $dataSet = new DataSet($dataSourceName, $row);
        $request = new DTO\Request\DataSet($dataSet, $method, $routePath, $userId);
        return $request;
    }

}
